import os
import requests
from tests.util import random_str
from datamaxi.api import API
from datamaxi.__version__ import __version__
import logging


def test_API_initial():
    """Tests the API initialization"""

    client = API()

    client.should.be.a(API)
    client.api_key.should.be.none
    client.timeout.should.be.none
    client.show_limit_usage.should.be.false
    client.show_header.should.be.false
    client.session.should.be.a(requests.Session)
    client.session.headers.should.have.key("Content-Type").which.should.equal(
        "application/json;charset=utf-8"
    )
    client.session.headers.should.have.key("User-Agent").which.should.equal(
        "datamaxi/" + __version__
    )
    client.session.headers.should.have.key("Authorization").which.should.be.equal(
        "X-DTMX-APIKEY None"
    )

    client._logger.should.be(logging.getLogger("datamaxi.api"))


def test_API_env_key():
    """Tests the API initialization with API key in environment variable"""

    api_key = random_str()
    os.environ["DATAMAXI_API_KEY"] = api_key
    client = API()

    client.should.be.a(API)
    client.api_key.should.be.equal(api_key)
    client.session.headers.should.have.key("Authorization").which.should.be.equal(
        f"X-DTMX-APIKEY {api_key}"
    )


def test_API_with_extra_parameters():
    """Tests the API initialization with extra parameters"""

    api_key = random_str()
    base_url = random_str()
    proxies = {"https": "https://1.2.3.4:8080"}

    client = API(
        api_key,
        base_url=base_url,
        show_limit_usage=True,
        show_header=True,
        timeout=0.1,
        proxies=proxies,
    )

    client.should.be.a(API)
    client.api_key.should.equal(api_key)
    client.timeout.should.equal(0.1)
    client.base_url.should.equal(base_url)
    client.show_limit_usage.should.be.true
    client.show_header.should.be.true
    client.proxies.should.equal(proxies)
    client.session.headers.should.have.key("Authorization").which.should.equal(
        f"X-DTMX-APIKEY {api_key}"
    )



from typing import Any, List, Union
import pandas as pd
from datamaxi.api import API
from datamaxi.lib.utils import check_required_parameter
from datamaxi.lib.constants import BASE_URL
from datamaxi.lib.utils import postprocess


class Naver(API):
    """Client to fetch Naver trend data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize the object.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        if "base_url" not in kwargs:
            kwargs["base_url"] = BASE_URL
        super().__init__(api_key, **kwargs)

    def keywords(self) -> List[str]:
        """Get Naver trend supported keywords

        `GET /v1/naver/keywords`

        <https://docs.datamaxi.finance/naver/keywords>

        Returns:
            List of supported Naver trend keywords
        """
        url_path = "/v1/naver/keywords"
        return self.query(url_path)

    @postprocess()
    def trend(self, keyword: str, pandas: bool = True) -> Union[List, pd.DataFrame]:
        """Get Naver trend for given keyword

        `GET /v1/naver/trend`

        <https://docs.datamaxiplus.com/naver/trend>

        Args:
            keyword (str): keyword to search for
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Naver trend data
        """
        check_required_parameter(keyword, "keyword")
        params = {"keyword": keyword}
        return self.query("/v1/naver/trend", params)



from typing import Any, List, Union
import pandas as pd
from datamaxi.api import API
from datamaxi.lib.utils import check_required_parameter
from datamaxi.lib.utils import check_required_parameters
from datamaxi.lib.utils import check_at_least_one_set_parameters
from datamaxi.lib.utils import postprocess
from datamaxi.lib.constants import BASE_URL


class Defillama(API):
    """Client to fetch Defillama data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize the object.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        if "base_url" not in kwargs:
            kwargs["base_url"] = BASE_URL
        super().__init__(api_key, **kwargs)

    def protocols(self) -> List[str]:
        """Get supported protocols

        `GET /v1/defillama/protocol`

        <https://docs.datamaxiplus.com/defillama/protocol>

        Returns:
            List of supported protocols
        """
        url_path = "/v1/defillama/protocol"
        return self.query(url_path)

    def chains(self) -> List[str]:
        """Get supported chains

        `GET /v1/defillama/chain`

        <https://docs.datamaxiplus.com/defillama/chain>

        Returns:
            List of supported chains
        """
        url_path = "/v1/defillama/chain"
        return self.query(url_path)

    def tokens(self) -> List[str]:
        """Get supported tokens

        `GET /v1/defillama/token`

        <https://docs.datamaxiplus.com/defillama/token>

        Returns:
            List of supported tokens
        """
        url_path = "/v1/defillama/token"
        return self.query(url_path)

    def pools(self) -> List[str]:
        """Get supported pools

        `GET /v1/defillama/pool`

        <https://docs.datamaxiplus.com/defillama/pool>

        Returns:
            List of supported pools
        """
        url_path = "/v1/defillama/pool"
        return self.query(url_path)

    def stablecoins(self) -> List[str]:
        """Get supported stablecoins

        `GET /v1/defillama/stablecoin`

        <https://docs.datamaxiplus.com/defillama/stablecoin>

        Returns:
            List of supported stablecoins
        """
        url_path = "/v1/defillama/stablecoin"
        return self.query(url_path)

    @postprocess()
    def tvl(
        self, protocol: str = None, chain: str = None, pandas: bool = True
    ) -> Union[List, pd.DataFrame]:
        """Get total TVL across all chains and protocols

        `GET /v1/defillama/tvl`

        <https://docs.datamaxiplus.com/defillama/tvl>

        Args:
            protocol (str): Protocol name
            chain (str): Chain name
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of total TVL
        """
        params = {}
        if protocol is not None:
            params["protocol"] = protocol

        if chain is not None:
            params["chain"] = chain

        url_path = "/v1/defillama/tvl"
        return self.query(url_path, params)

    @postprocess(num_index=2)
    def tvl_detail(
        self, protocol: str, chain: str = None, token: bool = False, pandas: bool = True
    ) -> Union[List, pd.DataFrame]:
        """Get TVL detail for given protocol and chain

        `GET /v1/defillama/tvl/detail`

        <https://docs.datamaxiplus.com/defillama/tvl-detail>

        Args:
            protocol (str): Protocol name
            chain (str): Chain name
            token (bool): Return token amount (return by default USD)
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of TVL detail for a given protocol and chain
        """
        check_required_parameter(protocol, "protocol")
        params = {"protocol": protocol}

        if chain is not None:
            params["chain"] = chain

        if token:
            params["token"] = str(token).lower()

        url_path = "/v1/defillama/tvl/detail"
        return self.query(url_path, params)

    @postprocess()
    def mcap(self, protocol: str, pandas: bool = True) -> Union[List, pd.DataFrame]:
        """Get market cap for given protocol

        `GET /v1/defillama/mcap`

        <https://docs.datamaxiplus.com/defillama/mcap>

        Args:
            protocols (str): Protocol name
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of market cap for given protocols
        """
        check_required_parameter(protocol, "protocol")
        params = {
            "protocol": protocol,
        }
        return self.query("/v1/defillama/mcap", params)

    @postprocess()
    def token_price(
        self, address: str, change: bool = False, pandas: bool = True
    ) -> Union[List, pd.DataFrame]:
        """Get token prices

        `GET /v1/defillama/token`

        <https://docs.datamaxiplus.com/defillama/token-price>

        Args:
            address (str): Token address
            change (bool): Return price change (default: False)
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of token prices
        """
        check_required_parameters([[address, "address"], [change, "change"]])
        params = {
            "address": address,
            "change": str(change).lower(),
        }
        return self.query("/v1/defillama/token", params)

    @postprocess()
    def pool_yield(self, poolId: str, pandas: bool = True) -> Union[List, pd.DataFrame]:
        """Get yield for given pool

        `GET /v1/defillama/pool/yield`

        <https://docs.datamaxiplus.com/defillama/yield>

        Args:
            poolId (str): Pool ID
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of yield for given pool
        """
        check_required_parameter(poolId, "poolId")
        params = {
            "poolId": poolId,
        }
        return self.query("/v1/defillama/pool/yield", params)

    @postprocess()
    def stablecoin_mcap(
        self, stablecoin: str, chain: str = None, pandas: bool = True
    ) -> Union[List, pd.DataFrame]:
        """Get market cap for given stablecoin and chain

        `GET /v1/defillama/stablecoin/mcap`

        <https://docs.datamaxiplus.com/defillama/stablecoin-mcap>

        Args:
            stablecoin (str): Stablecoin name
            chain (str): Chain name (optional)
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of market cap for given stablecoin and chain
        """
        check_required_parameter(stablecoin, "stablecoin")
        params = {
            "stablecoin": stablecoin,
        }

        if chain is not None:
            params["chain"] = chain

        return self.query("/v1/defillama/stablecoin/mcap", params)

    @postprocess()
    def stablecoin_price(
        self, stablecoin: str, pandas: bool = True
    ) -> Union[List, pd.DataFrame]:
        """Get price for given stablecoin

        `GET /v1/defillama/stablecoin/price`

        <https://docs.datamaxiplus.com/defillama/stablecoin-price>

        Args:
            stablecoin (str): Stablecoin name
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of stablecoin prices
        """
        check_required_parameter(stablecoin, "stablecoin")
        params = {
            "stablecoin": stablecoin,
        }
        print(params)
        return self.query("/v1/defillama/stablecoin/price", params)

    @postprocess()
    def fee(
        self,
        protocol: str = None,
        chain: str = None,
        daily: bool = True,
        pandas: bool = True,
    ) -> Union[List, pd.DataFrame]:
        """Get fee for given protocol or chain

        `GET /v1/defillama/fee`

        <https://docs.datamaxiplus.com/defillama/fee>

        Args:
            protocol (str): Protocol name
            chain (str): Chain name
            daily (bool): Daily fee or total fee
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of protocol or fees
        """
        check_required_parameter(daily, "daily")
        params = {
            "daily": str(daily).lower(),
        }

        if not ((protocol is None) ^ (chain is None)):
            raise ValueError("Either protocols or chains should be provided")

        if protocol is not None:
            check_required_parameter(protocol, "protocol")
            params["protocol"] = protocol
        elif chain is not None:
            check_required_parameter(chain, "chain")
            params["chain"] = chain

        return self.query("/v1/defillama/fee", params)

    @postprocess()
    def revenue(
        self,
        protocol: str = None,
        chain: str = None,
        daily: bool = True,
        pandas: bool = True,
    ) -> Union[List, pd.DataFrame]:
        """Get revenue for given protocol or chain

        `GET /v1/defillama/revenue`

        <https://docs.datamaxiplus.com/defillama/revenue>

        Args:
            protocol (str): Protocol name
            chain (str): Chain name
            daily (bool): Daily revenue or total revenue
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of protocol or chain revenues
        """
        check_required_parameter(daily, "daily")
        params = {
            "daily": str(daily).lower(),
        }

        if not ((protocol is None) ^ (chain is None)):
            raise ValueError("Either protocols or chains should be provided")

        if protocol is not None:
            check_required_parameter(protocol, "protocol")
            params["protocol"] = protocol
        elif chain is not None:
            check_required_parameter(chain, "chain")
            params["chain"] = chain

        return self.query("/v1/defillama/revenue", params)

    @postprocess(num_index=3)
    def fee_detail(
        self,
        protocol: str = None,
        chain: str = None,
        daily: bool = True,
        pandas: bool = True,
    ) -> Union[List, pd.DataFrame]:
        """Get fee detail for given protocol and chain

        `GET /v1/defillama/fee/detail`

        <https://docs.datamaxiplus.com/defillama/fee-detail>

        Args:
            protocol (str): Protocol name
            chain (str): Chain name
            daily (bool): Daily fee or total fee
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of fee detail for a given protocol and chain
        """
        check_required_parameter(daily, "daily")
        params = {
            "daily": str(daily).lower(),
        }

        check_at_least_one_set_parameters([[protocol, "protocol"], [chain, "chain"]])
        if protocol is not None:
            params["protocol"] = protocol

        if chain is not None:
            params["chain"] = chain

        return self.query("/v1/defillama/fee/detail", params)

    @postprocess(num_index=3)
    def revenue_detail(
        self,
        protocol: str = None,
        chain: str = None,
        daily: bool = True,
        pandas: bool = True,
    ) -> Union[List, pd.DataFrame]:
        """Get revenue detail for given protocol and chain

        `GET /v1/defillama/revenue/detail`

        <https://docs.datamaxiplus.com/defillama/revenue-detail>

        Args:
            protocol (str): Protocol name
            chain (str): Chain name
            daily (bool): Daily revenue or total revenue
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Timeseries of revenue detail for a given protocol and chain
        """
        check_required_parameter(daily, "daily")
        params = {
            "daily": str(daily).lower(),
        }

        check_at_least_one_set_parameters([[protocol, "protocol"], [chain, "chain"]])
        if protocol is not None:
            params["protocol"] = protocol

        if chain is not None:
            params["chain"] = chain

        return self.query("/v1/defillama/revenue/detail", params)



import os
import json
from json import JSONDecodeError
import logging
import requests
from .__version__ import __version__
from datamaxi.error import ClientError, ServerError
from datamaxi.lib.utils import cleanNoneValue
from datamaxi.lib.utils import encoded_string


class API(object):
    """The base class for all DataMaxi+ Python clients. `api_key` can be set
    as an environment variable `DATAMAXI_API_KEY`.
    """

    def __init__(
        self,
        api_key=None,
        base_url=None,
        timeout=None,
        proxies=None,
        show_limit_usage=False,
        show_header=False,
    ):
        """Client API constructor. `api_key` can be set
        as an environment variable `DATAMAXI_API_KEY`.

        Args:
            api_key (str): The API key for the DataMaxi+ API.
            base_url (str): The base URL for the DataMaxi+ API.
            timeout (int): The timeout for the requests.
            proxies (dict): The proxies for the requests.
            show_limit_usage (bool): Show the limit usage.
            show_header (bool): Show the header.
        """
        self.api_key = api_key or os.environ.get("DATAMAXI_API_KEY")
        self.base_url = base_url
        self.timeout = timeout
        self.proxies = None
        self.show_limit_usage = False
        self.show_header = False

        self.session = requests.Session()
        self.session.headers.update(
            {
                "Content-Type": "application/json;charset=utf-8",
                "User-Agent": "datamaxi/" + __version__,
                "Authorization": "X-DTMX-APIKEY " + str(self.api_key),
            }
        )

        if show_limit_usage is True:
            self.show_limit_usage = True

        if show_header is True:
            self.show_header = True

        if type(proxies) is dict:
            self.proxies = proxies

        self._logger = logging.getLogger(__name__)
        return

    def query(self, url_path, payload=None):
        return self.send_request("GET", url_path, payload=payload)

    def send_request(self, http_method, url_path, payload=None):
        if payload is None:
            payload = {}
        url = self.base_url + url_path
        self._logger.debug("url: " + url)
        params = cleanNoneValue(
            {
                "url": url,
                "params": self._prepare_params(payload),
                "timeout": self.timeout,
                "proxies": self.proxies,
            }
        )
        response = self._dispatch_request(http_method)(**params)
        self._logger.debug("raw response from server:" + response.text)
        self._handle_exception(response)

        try:
            data = response.json()
        except ValueError:
            data = response.text
        result = {}

        if self.show_limit_usage:
            limit_usage = {}
            for key in response.headers.keys():
                key = key.lower()
                if (
                    key.startswith("x-ratelimit-limit")
                    or key.startswith("x-ratelimit-remaining")
                    or key.startswith("x-ratelimit-reset")
                ):
                    limit_usage[key] = response.headers[key]
            result["limit_usage"] = limit_usage

        if self.show_header:
            result["header"] = response.headers

        if len(result) != 0:
            result["data"] = data
            return result

        return data

    def _prepare_params(self, params):
        return encoded_string(cleanNoneValue(params))

    def _dispatch_request(self, http_method):
        return {
            "GET": self.session.get,
            "DELETE": self.session.delete,
            "PUT": self.session.put,
            "POST": self.session.post,
        }.get(http_method, "GET")

    def _handle_exception(self, response):
        status_code = response.status_code
        if status_code < 400:
            return
        if 400 <= status_code < 500:
            try:
                err = json.loads(response.text)
            except JSONDecodeError:
                raise ClientError(
                    status_code, None, response.text, None, response.headers
                )
            error_data = None
            if "data" in err:
                error_data = err["data"]
            raise ClientError(
                status_code,
                err["code"],
                err["msg"],
                response.headers,
                error_data,
            )
        raise ServerError(status_code, response.text)
